@@ -9,7 +9,7 @@ from typing import Mapping, Tuple, Type
 from linkml_runtime.utils.namespaces import Namespaces
 from deprecated.classic import deprecated
 from linkml_runtime.utils.context_utils import parse_import_map, map_import
-from linkml_runtime.utils.pattern import generate_patterns
+from linkml_runtime.utils.pattern import PatternResolver
 from linkml_runtime.linkml_model.meta import *
 from enum import Enum
 logger = logging.getLogger(__name__)
@@ -450,19 +450,21 @@ class SchemaView(object):
             return c
 
     @lru_cache()
-    def get_slot(self, slot_name: SLOT_NAME, imports=True, attributes=False, strict=False) -> SlotDefinition:
+    def get_slot(self, slot_name: SLOT_NAME, imports=True, attributes=True, strict=False) -> SlotDefinition:
         """
         :param slot_name: name of the slot to be retrieved
         :param imports: include import closure
+        :param attributes: include attributes
+        :param strict: raise ValueError is not found
         :return: slot definition
         """
-        slot = self.all_slots(imports=imports).get(slot_name, None)
+        slot = self.all_slots(imports=imports, attributes=False).get(slot_name, None)
         if slot is None and attributes:
             for c in self.all_classes(imports=imports).values():
                 if slot_name in c.attributes:
                     if slot is not None:
-                        # slot name is ambiguous, no results
-                        return None
+                        # slot name is ambiguous: return a stub slot
+                        return SlotDefinition(slot_name)
                     slot = copy(c.attributes[slot_name])
                     slot.from_schema = c.from_schema
                     slot.owner = c.name
@@ -548,7 +550,10 @@ class SchemaView(object):
         :return: all direct parent slot names (is_a and mixins)
         """
         s = self.get_slot(slot_name, imports, strict=True)
-        return self._parents(s, imports, mixins, is_a)
+        if s:
+            return self._parents(s, imports, mixins, is_a)
+        else:
+            return []
 
     @lru_cache()
     def type_parents(self, type_name: TYPE_NAME, imports=True) -> List[TypeDefinitionName]:
@@ -1053,7 +1058,7 @@ class SchemaView(object):
             slot_anc_names = self.slot_ancestors(slot_name, reflexive=True)
             # inheritable slot: first propagate from ancestors
             for anc_sn in reversed(slot_anc_names):
-                anc_slot = self.get_slot(anc_sn)
+                anc_slot = self.get_slot(anc_sn, attributes=False)
                 for metaslot_name in SlotDefinition._inherited_slots:
                     if getattr(anc_slot, metaslot_name, None):
                         setattr(islot, metaslot_name, deepcopy(getattr(anc_slot, metaslot_name)))
@@ -1062,7 +1067,10 @@ class SchemaView(object):
             #    if getattr(slot, metaslot_name, None):
             #        setattr(islot, metaslot_name, deepcopy(getattr(slot, metaslot_name)))
         else:
-            raise ValueError(f'No such slot: {slot_name} and no attribute by that name in ancestors of {class_name}')
+            slot = self.get_slot(slot_name, imports, attributes=True)
+            if slot is None:
+                raise ValueError(f'No such slot: {slot_name} and no attribute by that name in ancestors of {class_name}')
+            islot = deepcopy(slot)
 
         COMBINE = {
             'maximum_value': lambda x, y: min(x, y),
@@ -1485,12 +1493,22 @@ class SchemaView(object):
         into regular expressions based on composite patterns 
         provided in the settings dictionary.
         """
-        patterns_dict = generate_patterns(self)
+        resolver = PatternResolver(self)
 
-        for _, slot_defn in self.all_slots().items():
-            if slot_defn.structured_pattern:
+        def materialize_pattern_into_slot_definition(slot_definition: SlotDefinition) -> None:
+            if not slot_definition.structured_pattern:
+                return
+            pattern = slot_definition.structured_pattern.syntax
+            slot_definition.pattern = resolver.resolve(pattern)
 
-                pattern = slot_defn.structured_pattern.syntax
+        for slot_definition in self.all_slots().values():
+            materialize_pattern_into_slot_definition(slot_definition)
 
-                if pattern in patterns_dict:
-                    slot_defn.pattern = patterns_dict[pattern]
+        for class_definition in self.all_classes().values():
+            if class_definition.slot_usage:
+                for slot_definition in class_definition.slot_usage.values():
+                    materialize_pattern_into_slot_definition(slot_definition)
+        
+            if class_definition.attributes:
+                for slot_definition in class_definition.attributes.values():
+                    materialize_pattern_into_slot_definition(slot_definition)
